-----------------------------------------------------------------------------
--  InterruptBar2 by Wardz. (Felesake) | Based on InterruptBar by Kollektiv.
-- 	Version 1.1.2 | General Public License v3.0 (GNU)
-- 	BETA
-----------------------------------------------------------------------------

-- Locals
local NAME, namespace = ...
local bar, IsInArena, IB = nil, false, CreateFrame('Frame')
local db, btns, classDetected, L = {}, {}, {}, namespace.L
local IB2Options

-- Cache frequently used globals in locals
local band, GetTime, pairs, _ = bit.band, GetTime, pairs, _
local COMBATLOG_OBJECT_REACTION_HOSTILE = COMBATLOG_OBJECT_REACTION_HOSTILE
local strformat, TFONT = string.format, STANDARD_TEXT_FONT
local playerGUID, OnUpdate = UnitGUID('player'), OnUpdate
local UnitInParty, SetFont = UnitInParty, SetFont
local SetCooldown, SetScript, SetTextColor = SetCooldown, SetScript, SetTextColor
local SetAlpha, GetFont, SetText = SetAlpha, GetFont, SetText

----------------------------------
-- Initialize saved variables
----------------------------------
function IB:SavedVariables()
	InterruptBar2DB = InterruptBar2DB or {}
	local characterInfo = UnitName('player') .. ' - ' .. GetRealmName()
	local defaultSettings = {
		smartArena = false,
		overlayDuration = 2,
		overlayFinish = false,
		overlayStart = true,
		abilityRestart = true,
		abilityMiss = true,
		resetAbilities = true,
		omniCC = true,
		spiralHide = false,
		barScale = 1.1,
		barHideMode = false,
		barSmartMode = false,
		barLock = false,
		barRows = 1,
		barTextures = true,
		barOffsetY = 33,
		barOffsetX = 33,
		barTransparent = false,
		barTransparentUse = false,
		barTransparency = 0.30,
		barTransparencyFiveLeft = 1,
		barTransparencyMinute = 1,
		timerHide = false,
		timerScale = 18,
		timerDecimal = false,
		timerOffsetX = 0,
		timerOffsetY = 0,
		timerColor = {
			r = 1,
			g = 1,
			b = 1,
			a = 1,
		},
		zoneBattle = false,
		zoneArena = false,
		abilities = {
		-- Unlocalized Class | Spell ID | Spell Cooldown
		-- Note: These are saved in your savedvariables (WTF folder)
		-- Changing these here will do nothing unless you reset your profile
			['MAGE'] = {
				[2139] = 24,
				[102051] = 20,
			},
			['WARLOCK'] = {
				[19647] = 24,
				[171139] = 24,
			},
			['WARRIOR'] = {
				[6552] = 15,
			},
			['DRUID'] = {
				[106839]=15,
				[78675] = 60,
			},
			['MONK'] = {
				[116705] = 15,
			},
			['SHAMAN'] = {
				[8177] = 25,
				[57994] = 12,
			},
			['PRIEST'] = {
				[15487] = 45,
			},
			['ROGUE'] = {
				[1766] = 15,
			},
			['HUNTER'] = {
				[147362] = 24,
			},
			['PALADIN'] = {
				[96231] = 15,
			},
			['DEATHKNIGHT'] = {
				[47528] = 15,
				[47476] = 60,
				[91802] = 30,
			},
		},
		announceFinish = false,
		announceFiveLeft = false,
		announceOnUse = false,
		timerScaleFiveLeft = 21,
		timerColorFiveLeft = {
			r = 1,
			g = 0,
			b = 0,
			a = 1,
		},
		timerScaleMinute = 10,
		timerColorMinute = {
			r = 0.4,
			g = 0.4,
			b = 0.4,
			a = 1,
		},
	}
	InterruptBar2DB[characterInfo] = InterruptBar2DB[characterInfo] or defaultSettings
	db = InterruptBar2DB[characterInfo]

	-- Wipe entire table if upgrading from older version, or profiles wont work properly
	if not ( db ) or ( InterruptBar2DB.version ) or ( InterruptBar2DB.scale ) then
		-- Check if old key values exists in table
		wipe(db)
		wipe(InterruptBar2DB)
		IB:SavedVariables()
	end

	if not ( db.Position ) then
		db.Position = {}
	end
end

----------------------------------
-- Core functions
----------------------------------

-- Retrieve RGB color code from table received
function IB:FormatColor(ColorTable)
	local r,g,b,a = ColorTable['r'], ColorTable['g'], ColorTable['b'], ColorTable['a']
	return r,g,b,a
end

-- Set font/text for btn
function IB:SetText(Frame, Colors, Size, FormatType, Minutes, Seconds)
	local fontName, fontHeight = Frame:GetFont()
	if not ( fontName ) or not ( fontHeight ) or ( floor(fontHeight+.5) ~= ceil(Size) ) then
		-- Only update font if something actually changed
		Frame:SetFont(TFONT, ceil(Size), 'OUTLINE')
		Frame:SetTextColor(IB:FormatColor(Colors)) -- To do: Compare this seperate
	end

	if ( fontName ) then -- If font is set
		if ( Minutes ) then
			-- Text for timers above 60sec
			Frame:SetText(strformat(FormatType, tonumber(Minutes), tonumber(Seconds) ))
		else
			-- Text for timers less than 60sec
			Frame:SetText(strformat(FormatType, tonumber(Seconds) ))
		end
	end
end

local lastMessage
-- Announce in chat for spell received
function IB:Announce(spellName, textMessage, channelType)
	if not ( UnitInParty('player') ) then
		return
	end
	if ( lastMessage ) and ( lastMessage - GetTime() > .2 ) then
		-- Throttle messages
		return
	end

	SendChatMessage(spellName..textMessage, channelType)
	lastMessage = GetTime()
end

-- Update visual effects on a specific time
function IB:UpdateFunctions(btn, cooldown)
	local getAlpha = btn:GetAlpha()
	if ( cooldown < 5 ) then
		if ( getAlpha ~= db.barTransparencyFiveLeft ) then
			-- Don't update unless something actually changed
			btn:SetAlpha(db.barTransparencyFiveLeft)
		end
		--IB:Announce(btn.name,' READY IN 5!', 'PARTY') Removed temporary. Will spam chat
	elseif ( cooldown > 60 ) then
		if ( getAlpha ~= db.barTransparencyMinute ) then
			btn:SetAlpha(db.barTransparencyMinute)
		end
	end
end

-- Timer text handler
function IB:UpdateTimerText(text, CooldownTime, btn)
	local cooldown
	local numbers
	if not ( db.timerDecimal ) then
		cooldown = floor(CooldownTime * 10 ^ 1  +.5)  / 10 / 1
	else
		cooldown = floor(CooldownTime + 0.5)
	end

	local minutes = floor(cooldown / 60)
	if ( minutes > 0 ) then
		-- Show minutes and seconds
		local seconds = cooldown - minutes * 60
		if ( minutes < 1 ) then
			numbers = '%s'
		else
			numbers = '%s:%.0f'
		end

		IB:SetText(text,db.timerColorMinute,db.timerScaleMinute,numbers,minutes,seconds,btn)
	else
		if ( cooldown > 5 ) then
			-- CD is higher than 5s and below 60s
			if ( db.timerDecimal ) then
				numbers = '%d'
			else
				numbers = '%.0f'
			end
			IB:SetText(text,db.timerColor,db.timerScale,numbers,_,cooldown,btn)
		else
			-- CD have less than 5s left
			numbers = '%.0f'
			IB:SetText(text,db.timerColorFiveLeft,db.timerScaleFiveLeft,numbers,_,cooldown,btn)

			if ( cooldown == 0 ) then
				text:SetText('')
			else
				if ( not db.timerDecimal ) then
					if not ( cooldown >= 1 ) then
						numbers = '.' .. floor(cooldown * 10)
					else
						numbers = '%.1f'
					end
				else
					numbers = '%d'
				end

				text:SetText(strformat(numbers, cooldown))
			end
		end
	end
end

-- Activate button received
function IB:Activate(...)
	local btn, start, miss = ...

	-- Restart or not if already active
	if ( btn.active ) then
		if not ( db.abilityRestart ) then
			return
		else
			IB:Deactivate(...)
			IB:Activate(...)
		end
	end

	-- Show if hidden
	if ( db.barHideMode ) then
		btn:Show()
	end

	-- Show overlay if SPELL_MISSED
	if ( miss ) then
		if ( db.barTextures ) then
			btn:SetBackdropColor(0.4, 0, 0.4, 1)
		else 
			btn.texture:SetVertexColor(0.4, 0, 0.4, 1)
		end
	else
		if ( db.barTextures ) then
			btn:SetBackdropColor(1, 1, 1, 1)
		else
			btn.texture:SetVertexColor(1, 1, 1, 1)
		end
	end

	-- Show timer text
	if not ( db.timerHide ) then
		btn.text:SetAlpha(1)
	end

	local cooldown = btn.duration - ( GetTime() - start )

	-- Start cooldown spiral
	if not ( db.spiralHide ) then
		btn.cd:Show()
		btn.cd:SetCooldown(start, cooldown)
	end

	btn.active = true
	if ( db.barHideMode ) then
		IB:UpdatePositions() -- Smart-Mode
	end

	-- Activation Border
	if ( db.overlayDuration ~= 0 ) and ( db.overlayStart ) then
		UIFrameFadeOut(btn.overlay, db.overlayDuration, 1, 0)
	end

	-- Set transparent on use
	if not ( db.barHideMode ) then
		if ( db.barTransparentUse ) then
			btn:SetAlpha(db.barTransparency)
		else
			btn:SetAlpha(1)
		end
	end

	-- Announce spell on use
	if ( db.announcePartyuse ) then
		IB:Announce(btn.name, ' USED!', 'PARTY')
	end

	-- OnUpdate
	-- Temporary timer
	btn:SetScript('OnUpdate', function(self, elapsed)
		cooldown = cooldown - elapsed

		if ( cooldown <= 0 ) then
			IB:Deactivate(btn)
		else
			if not ( db.timerHide ) then
				IB:UpdateTimerText(btn.text, cooldown, btn)
			end

			IB:UpdateFunctions(btn, cooldown)
		end
	end)

end

-- Deactivate button received
function IB:Deactivate(btn, start, miss)
	if not ( db.spiralHide ) then
		-- For animation effect
		btn.cd:SetCooldown(0, 0)
	end

	-- Hide if hide mode
	if ( db.barHideMode ) then
		btn:Hide()
	end

	if ( not db.timerHide ) then
		-- Hide timer text
		btn.text:SetAlpha(0)
	end

	btn.active = false
	btn:SetScript('OnUpdate', nil)
	if ( db.barHideMode ) then
		IB:UpdatePositions() -- Smart-Mode
	end

	if ( db.announcePartyfinish ) then
		IB:Announce(btn.name,' READY!','PARTY')
	end


	-- Transparent on end
	if ( not db.barHideMode ) then
		if ( db.barTransparent ) then
			btn:SetAlpha(db.barTransparency)
		else
			btn:SetAlpha(1)
		end
	end

	-- Activation Border
	if ( db.overlayDuration ~= 0 ) and ( db.overlayFinish ) then
		UIFrameFadeOut(btn.overlay, db.overlayDuration, 1, 0)
	end
end

-- Create frame for every spell info received
function IB:CreateIcon(spellID, spellCD)
	local btn = CreateFrame('Frame', nil, bar)
	local _,_,spellIcon = GetSpellInfo(spellID)

	-- Cooldown Spiral
	btn.cd = CreateFrame('Cooldown', nil, btn, 'CooldownFrameTemplate')
	btn.cd:SetAllPoints(btn)
	btn.cd.noomnicc = true
	btn.cd:SetDrawSwipe(not db.spiralHide)
	btn.cd:SetHideCountdownNumbers(true)
	btn.cd:Hide()

	-- Icon
	if ( db.barTextures ) then 
		btn:SetBackdrop({
			bgFile = spellIcon,
			edgeFile = 'Interface\\Tooltips\\UI-Tooltip-Border',
			tileSize = 30 + 2,
			edgeSize = 0.3 * 30,
		})
		btn:SetBackdropBorderColor(0.4, 0.4, 0.4)
	else
		btn.texture = btn:CreateTexture(nil, 'BORDER')
		btn.texture:SetAllPoints(btn)
		btn.texture:SetTexture(spellIcon)
		btn.texture:SetTexCoord(0.07, 0.9, 0.07, 0.90)
	end

	-- Activation Border
	if ( db.overlayDuration ~= 0 ) and ( db.overlayFinish ) or ( db.overlayStart ) then
		btn.overlay = btn:CreateTexture(nil, 'OVERLAY')
		btn.overlay:SetAllPoints(btn)
		btn.overlay:SetTexture('Interface/SpellActivationOverlay/IconAlert')
		btn.overlay:SetTexCoord(0.00781250, 0.50781250, 0.27734375, 0.52734375)
		btn.overlay:SetPoint("TOPLEFT", btn, "TOPLEFT", -30 * 0.4, 30 * 0.4)
		btn.overlay:SetPoint("BOTTOMRIGHT", btn, "BOTTOMRIGHT", 30 * 0.4, -30 * 0.4)
		btn.overlay:SetAlpha(0)
	end

	-- Save spell name once for announcements
	btn.name = GetSpellInfo(spellID)

	-- Text Timer
	if not ( db.timerHide ) then
		btn.text = btn:CreateFontString(nil, 'ARTWORK')
		btn.text:SetAlpha(0)
		btn.text:SetPoint('CENTER', btn, 'CENTER', db.timerOffsetX, db.timerOffsetY)
	end

	btn:SetSize(30, 30)
	btn:SetFrameStrata('LOW')
	btn.duration = spellCD
	btn.spellID = spellID

	return btn
end

-- Create icon only if arena opponent class matches ability class (untested)
function IB:DetectArena()
	if not ( IsInArena ) then
		return
	end

	local numOpps = GetNumArenaOpponentSpecs()
	local offset, height = db.barOffsetX, db.barRows
	local y, x, count = -45, 0, 0

	for i = 1, MAX_ARENA_ENEMIES do
		if ( i <= numOpps ) then
			local specID = GetArenaOpponentSpec(i)
			if ( specID > 0 ) then
				local _,_,_,_,_,_, enemyClass = GetSpecializationInfoByID(specID)
				if ( enemyClass ) then
					for class, table in pairs(db.abilities) do
						if enemyClass == class then
							for ability,cooldown in pairs(table) do
								if btns[ability] then return end
								local btn = IB:CreateIcon(ability, cooldown)
								btns[ability] = btn
								if ( count > height ) then
									y = -45
									x = x + offset
									count = 0
								end

								btn:SetPoint('CENTER', bar, 'CENTER', x, y)

								y = y + db.barOffsetY
								count = count + 1
							end
						end
					end
				end
			end
		end
	end
end

function IB:AddIcons()
	local offset, height = db.barOffsetX, db.barRows
	local y, x, count = -45, 0 , 0

	if not ( IsInArena ) then
		for _, table in pairs(db.abilities) do
			for ability, cooldown in pairs(table) do
				local btn = IB:CreateIcon(ability, cooldown)

				btns[ability] = btn
				if ( count > height ) then
					y = -45
					x = x + offset
					count = 0
				end

				btn:SetPoint('CENTER', bar, 'CENTER', x, y)

				y = y + db.barOffsetY
				count = count + 1
			end
		end
	end
end


-- Update positons every button Activate() or Deactivate()
function IB:UpdatePositions()
	if not ( db.barSmartMode ) then
		return
	end
	local offset,height = db.barOffsetX, db.barRows
	local y, x, count = -45, 0, 0

	for _,btn in pairs(btns) do
		if ( btn.active ) then
			if ( count > height ) then
				y = -45
				x = x + offset
				count = 0
			end

			btn:SetPoint('CENTER', bar, 'CENTER', x, y)

			y = y + db.barOffsetY
			count = count + 1
		end
	end
end

-- Save frame bar position
function IB:SavePosition(self)
	local point, _, relativePoint, xOfs, yOfs = self:GetPoint()
	db.Position.point,db.Position.relativePoint,db.Position.xOfs,db.Position.yOfs = point,relativePoint,xOfs,yOfs
end

-- Create main bar frame
function IB:CreateBar()
	bar = CreateFrame('Frame', nil, UIParent)
	bar:SetClampedToScreen(true)
	bar:EnableMouse(true)
	bar:SetMovable(true)
	bar:SetScale(db.barScale)

	-- Start moving
	bar:SetScript('OnMouseDown', function(self)
		if ( db.barLock ) then
			return
		end
		self:StartMoving()
	end)

	-- Stop moving
	bar:SetScript('OnMouseUp', function(self,button)
		if ( button == 'RightButton' ) then
			if ( db.barLock ) then
				return
			end
			db.barLock = true
			IB:Restart()
		end
		self:StopMovingOrSizing()
		IB:SavePosition(self)
	end)

	bar:SetWidth(200)
	bar:SetHeight(25)

	-- Text tooltip
	bar.Text = bar:CreateFontString(nil, 'ARTWORK')
	bar.Text:SetPoint('TOP', bar, 'TOP', 0, -6)
	bar.Text:SetFont(TFONT, 10 , 'OUTLINE')
	bar.Text:SetText('Left Click - Move | Right Click - Lock')
	bar:SetBackdrop({bgFile = 'Interface\\ChatFrame\\ChatFrameBackground', edgeFile='Interface\\Tooltips\\UI-Tooltip-Border',tileSize=30+2,edgeSize=.3*30,})
	bar:Show()

	if ( db.Position.point ) then
		bar:SetPoint(db.Position.point,UIParent,db.Position.relativePoint,db.Position.xOfs,db.Position.yOfs)
	else
		-- Default position
		bar:SetPoint('CENTER', UIParent, 'CENTER')
	end

	-- Create spell icons
	IB:AddIcons()
	IB:UpdateFrame()
end

----------------------------------
-- Combat Log 
----------------------------------

-- Redirect spell X to spell Y
-- Will add more further into WoD
local spellRedirects = {
	[132409] = 19647,
	[115781] = 19647,
}

-- Cooldown reseters
local ResetsSnap = {
	12043,
	31661,
	120,
	122,
	45438,
}
local ResetsPrep = {
	2983,
	1856,
	5277,
}
local ResetsCotE = {
	108269,
	8177,
	108270,
	108273,
	51485,
	8143,
	5394,
	2484,
}

local ColdSnap, Prep, CotE = 11958, 14185, 108285
local GUID_INVALID = '0x0000000000000000'

function IB:CLEU(event, ...)
	local _, eventType, _, srcGUID, _, srcFlags, _, _, _, _, _, id = ...
	if not ( srcGUID ) or ( srcGUID == GUID_INVALID ) or ( id == 0 ) then return end
	if not ( band(srcFlags, COMBATLOG_OBJECT_REACTION_HOSTILE) ~= 0 )  then return end
	if ( btns[id] ) or spellRedirects[id] or ( id == 11958 or id == 14185 or id == 108285 ) then

	local start = start or GetTime()

	if ( eventType == 'SPELL_CAST_SUCCESS' ) then
		if ( db.resetAbilities ) then
			if ( id == ColdSnap ) then -- Should be faster than table lookups
				for k,_ in pairs(ResetsSnap) do
					local btn = btns[k]
					if ( btn ) then
						IB:Deactivate(btn, start)
					end
				end
			elseif ( id == Prep ) then
				for k,_ in pairs(ResetsPrep) do
					local btn = btns[k]
					if ( btn ) then
						IB:Deactivate(btn, start)
					end
				end
			elseif ( id == CotE ) then
				for k,_ in pairs(ResetsCotE) do
					local btn = btns[k]
					if ( btn ) then
						IB:Deactivate(btn, start)
					end
				end
			end
		end

		if ( spellRedirects[id] ) then
			for k,v in pairs (spellRedirects) do
				if k == id then
					id = v
				end
			end
		end

		local btn = btns[id]
		IB:Activate(btn, start)
		return
	end

	if ( db.abilityMiss ) then
		if ( eventType == 'SPELL_MISSED' ) then
			local btn, miss = btns[id], true
			IB:Activate(btn,start,miss)
			return
		end
	end
	end
end

----------------------------------
-- Restart/Update
----------------------------------

-- Update settings that doesn't require recreating a frame
function IB:UpdateFrame()
	for _,btn in pairs(btns) do
		IB:Deactivate(btn)
		if ( db.barHideMode ) then
			btn:Hide()
		else
			btn:Show()
		end

		if ( db.barTransparent ) and ( not db.barHideMode ) then
			btn:SetAlpha(db.barTransparency)
		else
			btn:SetAlpha(1)
		end
	end

	--
	bar:SetScale(db.barScale)

	--
	if ( db.barLock ) then
		bar:EnableMouse(false)
		bar.Text:Hide()
		bar:SetBackdropColor(0,0,0,0)
		bar:SetBackdropBorderColor(0,0,0,0)
	else
		bar:EnableMouse(true)
		bar.Text:Show()
		bar:SetBackdropColor(0,0,0,.7)
		bar:SetBackdropBorderColor(1,1,1,1)
	end
end

-- Update setting that does require recreating a frame
function IB:Restart()
	for _,btn in pairs(btns) do
		if not ( btn ) then return end
		if ( btn.active ) then
			IB:Deactivate(btn)
		end
		btn:Hide()
		btn:SetParent(nil)
	end
	wipe(btns)
	--bar:Hide()
	IB:AddIcons()
	IB:UpdateFrame()
end

----------------------------------
-- Options
----------------------------------

-- Create button
local function CreateButton(text, parent, pos, x, y)
	local button = CreateFrame('Button', nil, parent, 'UIPanelButtonTemplate')
	button:SetWidth(85)
	button:SetHeight(30)
	button.text = button:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	button.text:SetPoint('CENTER', button, 'CENTER', 1, -1)
	button.text:SetText(text)
	button:SetPoint(pos,parent,x,y)
	return button
end

function IB:LoadOptions()
	-- Create slider
	local function CreateSlider(text, parent, low, high, step, value, pos, x, y)
		local name = 'IB2:'..text
		local slider = CreateFrame('Slider', name, parent, 'OptionsSliderTemplate')
		slider:SetWidth(145)
		slider:SetMinMaxValues(low, high)
		slider:SetObeyStepOnDrag(true)
		slider:SetValueStep(step)
		slider:SetValue(value)
		slider:SetPoint(pos,parent,x,y)
		_G[name..'Text']:SetText(text)
		_G[name..'Low']:SetText(low)
		_G[name..'High']:SetText(high)
		return slider
	end

	-- Create checkbox
	local function CreateCheckBox(text,parent,value,pos,x,y)
		local name = 'IB2:'..text
		local checkbox = CreateFrame('CheckButton', name, parent, 'ChatConfigCheckButtonTemplate')
		_G[checkbox:GetName()..'Text']:SetText(text)
		if value then checkbox:SetChecked(true) else checkbox:SetChecked(false) end
		checkbox:SetPoint(pos,parent,x,y)
		return checkbox
	end

	-- Create text
	local function CreateText(textmsg,parent,tsize,pos,x,y)
		local text = parent:CreateFontString(nil, 'ARTWORK')
		text:SetFont(TFONT, tsize)
		text:SetText(textmsg)
		text:SetPoint(pos,parent,pos,x,y)
		return text
	end

	local function FormatClassName(getclassName)
		getclassName = getclassName:gsub("(%a)(%a*)('*)(%a*)", function (a,b,c,d) return a:upper()..b:upper()..c..d:upper() end)
		getclassName = getclassName:gsub('%s+', '')

		return getclassName
	end

	-- Find spell ID by spell Name
	local function FormatSpellName(getSpell)
		getSpell = getSpell:gsub("(%a)(%a*)('*)(%a*)", function (a,b,c,d) return a:upper()..b:lower()..c..d:lower() end)
		getSpell = getSpell:gsub('(The)', string.lower)
		getSpell = getSpell:gsub('(Of)', string.lower)

		--[[if ( GetSpellInfo(getSpell) ) then
			-- Check if spell exists in spellbook
			return getSpell
		end]]

		for i = 1, 130000 do
			if ( getSpell == GetSpellInfo(i) ) then
				return i
			end
		end
	end

	local playerName = UnitName('player')..' - '..GetRealmName()
	

	-- Title
	IB2Options.T1 = CreateText(L['InterruptBar2 Settings'], IB2Options, 20, 'TOP', -190, -20)

	-- Test button
	IB2Options.BTN1 = CreateButton(L['Test'], IB2Options, 'BOTTOM', 250, 13)
	IB2Options.BTN1:SetScript('OnClick', function()
		for _,btn in pairs(btns) do
			local start = start or GetTime()
		--	IB:Deactivate(btn)
			IB:Activate(btn,start)
		end
	end)

	IB2Options.BTN2 = CreateButton(L['Unlock bar'], IB2Options, 'BOTTOM', 250, 50)
	IB2Options.BTN2:SetScript('OnClick', function()
		db.barLock = not db.barLock
		IB:UpdateFrame()
	end)

	-- Reset Button
	IB2Options.default = function()
		wipe(InterruptBar2DB[playerName])
		wipe(db)
		IB:RegisterEvent('PLAYER_LOGIN')
		ReloadUI()
	end

	IB2Options.refresh = function()
		IB2Options.SL1:SetValue(db.barOffsetX)
		IB2Options.SL2:SetValue(db.barOffsetY)
		IB2Options.SL3:SetValue(db.barScale)
		IB2Options.SL4:SetValue(db.barRows)
		IB2Options.SL5:SetValue(db.barTransparency)
		if db.barTextures then IB2Options.CB2:SetChecked(true) else IB2Options.CB2:SetChecked(false) end
		if db.zoneBattle then IB2Options.CB3:SetChecked(true) else IB2Options.CB3:SetChecked(false) end
		if db.barHideMode then IB2Options.CB4:SetChecked(true) else IB2Options.CB4:SetChecked(false) end
		if db.barTransparent then IB2Options.CB5:SetChecked(true) else IB2Options.CB5:SetChecked(false) end
		if db.barTransparentUse then IB2Options.CB6:SetChecked(true) else IB2Options.CB6:SetChecked(false) end
		if db.barSmartMode then IB2Options.CB7:SetChecked(true) else IB2Options.CB7:SetChecked(false) end
		if db.zoneArena then IB2Options.CB8:SetChecked(true) else IB2Options.CB8:SetChecked(false) end
		if db.smartArena then IB2Options.CB9:SetChecked(true) else IB2Options.CB9:SetChecked(false) end
	end

	-- Text 2
	IB2Options.T2 = CreateText(L['Distance between icons'], IB2Options, 12, 'TOP', -215, -70)

	-- Frame offset X
	IB2Options.SL1 = CreateSlider(L['Offset X'], IB2Options, 30, 100, 1, db.barOffsetX, 'LEFT', 30, 165)
	IB2Options.SL1:SetScript('OnValueChanged', function(self, value)
		_G[self:GetName() .. 'Text']:SetText(L['Offset X'] .. ' (' .. floor(value) .. ')')
		db.barOffsetX = tonumber(value)
		IB:Restart()
	end)

	-- Frame offset Y
	IB2Options.SL2 = CreateSlider(L['Offset Y'], IB2Options.SL1, 30, 100, 1, db.barOffsetY, 'LEFT', 210, 0)
	IB2Options.SL2:SetScript('OnValueChanged', function(self, value)
		if ( db.barRows < 0 ) then
			return
		else
			_G[self:GetName() .. 'Text']:SetText(L['Offset Y'] .. ' (' .. floor(value) .. ')')
			db.barOffsetY = tonumber(value)
			IB:Restart()
		end
	end)

	-- Text 3
	IB2Options.T3 = CreateText(L['Icon size and columns'], IB2Options, 12, 'TOP', -215, -160)

	-- Icon scale
	IB2Options.SL3 = CreateSlider(L['Icon Scale'], IB2Options, 0.1, 5, 0.1, db.barScale, 'LEFT', 30, 80)
	IB2Options.SL3:SetScript('OnValueChanged', function(self, value)
		_G[self:GetName() .. 'Text']:SetText(L['Icon Scale'] .. ' (' .. floor(value) .. ')')
		db.barScale = tonumber(value)
		IB:UpdateFrame()
	end)

	-- Rows
	IB2Options.SL4 = CreateSlider(L['Bar Rows'], IB2Options.SL3, 0, 25, 1, db.barRows, 'LEFT', 210, 0)
	IB2Options.SL4:SetScript('OnValueChanged', function(self, value)
		_G[self:GetName() .. 'Text']:SetText(L['Bar Rows'] .. ' (' .. value .. ')')
		db.barRows = tonumber(value)
		IB:Restart()
	end)

	-- Text 4
	IB2Options.T4 = CreateText(L['Misc Settings'], IB2Options, 12, 'TOP', -243, -250)

	-- Border Textures
	IB2Options.CB2 = CreateCheckBox(L['Show borders'], IB2Options, db.barTextures, 'LEFT', 40, 0)
	IB2Options.CB2:SetScript('OnClick', function()
		db.barTextures = not db.barTextures
		IB:Restart()
	end)

	-- Disable in BattleGrounds
	IB2Options.CB3 = CreateCheckBox(L['Battlegrounds'], IB2Options, db.zoneBattle, 'TOP', 175, -55)
	IB2Options.CB3:SetScript('OnClick', function()
		db.zoneBattle = not db.zoneBattle
		IB:Restart()
	end)

	-- Hide Mode
	IB2Options.CB4 = CreateCheckBox(L['Hide-Mode'], IB2Options, db.barHideMode, 'LEFT', 40, -30)
	IB2Options.CB4:SetScript('OnClick', function()
		db.barHideMode = not db.barHideMode
		if ( db.barTransparent ) and ( db.barHideMode ) then
			db.barTransparent = false
		end
		IB:UpdateFrame()
	end)

	-- Text 5
	IB2Options.T5 = CreateText(L['Transparent icons'], IB2Options, 12, 'LEFT', 35, -90)

	-- Transparent Slider
	IB2Options.SL5 = CreateSlider(L['Transparency amount'], IB2Options, 0, 1, .1, db.barTransparency, 'LEFT', 40, -125)
	IB2Options.SL5:SetScript('OnValueChanged', function(self, value)
		_G[self:GetName() .. 'Text']:SetText(L['Transparency amount'] .. ' (' .. floor(value) .. ')')
		db.barTransparency = tonumber(value)
		IB:UpdateFrame()
	end)

	-- Transparent spell over
	IB2Options.CB5 = CreateCheckBox(L['On spell finished'], IB2Options, db.barTransparent, 'LEFT',40,-160)
	IB2Options.CB5:SetScript('OnClick', function()
		db.barTransparent = not db.barTransparent
		if ( db.barTransparent ) and ( db.barHideMode ) then
			db.barHideMode = false
		end
		IB:UpdateFrame()
	end)

	-- Transparent spell used
	IB2Options.CB6 = CreateCheckBox(L['On spell used'], IB2Options, db.barTransparentUse, 'LEFT', 40,-180)
	IB2Options.CB6:SetScript('OnClick', function()
		db.barTransparentUse = not db.barTransparentUse
		if ( db.barTransparentUse ) and ( db.barHideMode ) then
			db.barHideMode = false
		end
	end)

	-- Text 5
	IB2Options.TA5 = CreateText(L['Activation border'], IB2Options, 12, 'LEFT', 305, -90)

	-- overlay duration slider
	IB2Options.SLA5 = CreateSlider(L['Duration'], IB2Options, 0, 4, .1, db.overlayDuration, 'LEFT', 310, -125)
	IB2Options.SLA5:SetScript('OnValueChanged', function(self, value)
		_G[self:GetName() .. 'Text']:SetText(L['Duration'] .. ' (' .. floor(value) .. ')')
		db.overlayDuration = tonumber(value)
	end)

	-- overlay finish
	IB2Options.CBA5 = CreateCheckBox(L['Show on spell finished'], IB2Options, db.overlayFinish, 'LEFT',310,-160)
	IB2Options.CBA5:SetScript('OnClick', function()
		db.overlayFinish = not db.overlayFinish
	end)

	-- overlay used
	IB2Options.CBA6 = CreateCheckBox(L['Show on spell used'], IB2Options, db.overlayStart, 'LEFT', 310,-180)
	IB2Options.CBA6:SetScript('OnClick', function()
		db.overlayStart = not db.overlayStart
	end)

	-- Smart Mode
	IB2Options.CB7 = CreateCheckBox(L['Smart-Mode'], IB2Options.CB2, db.barSmartMode, 'LEFT',190,0)
	IB2Options.CB7:SetScript('OnClick', function()
		db.barSmartMode = not db.barSmartMode
		IB:Restart()
	end)

	-- Arena Only
	IB2Options.CB8 = CreateCheckBox(L['Arena Only'], IB2Options, db.zoneArena,'TOP', 175,-85)
	IB2Options.CB8:SetScript('OnClick', function()
		db.zoneArena = not db.zoneArena
		local _,instance = IsInInstance()
		IB:PLAYER_ENTERING_WORLD(instance)
	end)

	-- Arena Only
	IB2Options.CB9 = CreateCheckBox(L['Show spells only if your arena opponents\nhave them (test)'], IB2Options, db.zoneArena,'LEFT', 230,-40)
	IB2Options.CB9:SetScript('OnClick', function()
		db.smartArena = not db.smartArena
		--
	end)

	-- Text 7
	IB2Options.T7 = CreateText(L['Enabled in'], IB2Options, 12,'TOP',190,-30)

	----------------------
	-- Add new spell
	----------------------
	local IB2OptionsSpell = CreateFrame('Frame')
	IB2OptionsSpell.name = L['Spell List']
	IB2OptionsSpell.parent = IB2Options.name

	-- Title text
	IB2OptionsSpell.T1 = CreateText(L['InterruptBar2 Spells'], IB2OptionsSpell, 20,'TOP', -162, -20)

	-- Text 8
	IB2OptionsSpell.T8 = CreateText(L['Add new spell'], IB2OptionsSpell, 12,'LEFT',36,200)

	-- Text 9
	IB2OptionsSpell.T9 = CreateText(L['Spell Name or Spell ID'], IB2OptionsSpell, 9,'LEFT',25, 180)

	-- Spell EditBox
	IB2OptionsSpell.EB1 = CreateFrame('EditBox', 'IB:EB1', IB2OptionsSpell, 'InputBoxTemplate')
	IB2OptionsSpell.EB1:SetPoint('LEFT', 30, 160)
	IB2OptionsSpell.EB1:SetSize(100,50)
	IB2OptionsSpell.EB1:SetAutoFocus(false)

	-- Text 10
	IB2OptionsSpell.T9 = CreateText(L['Class name'], IB2OptionsSpell, 10,'LEFT', 175, 180)

	IB2OptionsSpell.T12 = CreateText(L['Example: Blink | Mage'], IB2OptionsSpell, 14,'LEFT', 55, 130)

	-- Class EditBox
	IB2OptionsSpell.EB2 = CreateFrame('EditBox', 'IB:EB2', IB2OptionsSpell, 'InputBoxTemplate')
	IB2OptionsSpell.EB2:SetPoint('LEFT', 180, 160)
	IB2OptionsSpell.EB2:SetSize(100,50)
	IB2OptionsSpell.EB2:SetAutoFocus(false)

	-- Add btn
	IB2OptionsSpell.BTN3 = CreateButton(L['Add'], IB2OptionsSpell.EB1, 'BOTTOM', 280, 13)
	IB2OptionsSpell.BTN3:SetScript('OnClick', function()
		local getSpell = IB2OptionsSpell.EB1:GetText()
		local getclassName = IB2OptionsSpell.EB2:GetText()
			
		if ( getSpell and getSpell ~= '' ) and getclassName ~= '' then
			local spellCD = nil
			local spellName = nil
			local getclassName = FormatClassName(getclassName)

			if not getclassName then
				print(L['IB2: Incorrect class name.'])
				return
			end
			
			if ( tonumber(getSpell) ) then
				-- Input is not string
				spellName = tonumber(getSpell)
				spellCD = GetSpellBaseCooldown(spellName)

				if ( spellCD ) then
					spellCD = spellCD / 1000
				else
					print(L['IB2: Invalid input or spell.'])
					return
				end
			else -- Input is string
				spellName = FormatSpellName(getSpell) -- Convert spell name to spell id
				if ( spellName ) then
					if ( spellName == 2066 ) then spellName = 22812 end
					spellCD = GetSpellBaseCooldown(spellName)

					if ( spellCD ) then
						spellCD = spellCD / 1000
					else
						-- Invalid spell ID sent to GetSpellBaseCooldown
						print(L['IB2: Invalid input or spell.'])
						return
					end
				else
					-- Couldn't convert spell name to spell id
					print(L['IB2: Invalid input or spell.'])
					return
				end
			end

			if ( spellName ) and ( spellCD ) and ( getclassName ) then
				if not ( db.abilities[getclassName][spellName] ) then
					if ( spellCD < 2 ) then
						print(L['IB2: Too short cooldown for spell: ']..spellName)
						return
					end


					db.abilities[getclassName][spellName] = spellCD
					InterfaceAddOnsList_Update()


					IB:Restart()
					IB2OptionsSpell.EB1:SetText('')

					print('IB2: '..spellName..L[' successfully added with cooldown: ']..spellCD)
				else
					print('IB2: '..spellName..L[' already exists.'])
				end
			end
		end
	end)

	IB2OptionsSpell.T3 = CreateText(L['Click on a spell icon to DELETE it from the bar'], IB2OptionsSpell, 12,'LEFT',20,100)

	local tablee = db.abilities
	local ix, yx = 30, 60
	for class,ntable in pairs(tablee) do
		for spell,cd in pairs(ntable) do
			local frame = CreateFrame("Frame",nil,IB2OptionsSpell)
			frame:EnableMouse(true)
			local _,_,icon = GetSpellInfo(spell)
			frame.tex = frame:CreateTexture(nil,'OVERLAY')
			frame.tex:SetTexture(icon)
			frame.tex:SetSize(30,30)
			frame:SetSize(30,30)
			frame.tex:SetAllPoints(frame)
			frame:SetScript('OnMouseDown', function(self,button)
				db.abilities[class][spell] = nil
				frame:Hide()
				print('IB2: '..spell..L[' successfully removed.'])
				IB:Restart()
			end)
			frame:SetPoint('LEFT', IB2OptionsSpell, 'LEFT', ix, yx)
			frame:Show()
			cacheFrame = frame

			yx = yx - 30
			if yx < -260 then
				ix = ix + 30
				yx = 60
			end
		end
	end


	-- << TIMER >>
	local IB2OptionsTimer = CreateFrame('Frame')
	IB2OptionsTimer.name = L['Timer Settings']
	IB2OptionsTimer.parent = IB2Options.name
	IB2OptionsTimer.refresh = function()
		IB2OptionsTimer.SL1:SetValue(db.timerScale)
		IB2OptionsTimer.SL2:SetValue(db.timerOffsetX)
		IB2OptionsTimer.SL3:SetValue(db.timerOffsetY)
		IB2OptionsTimer.SL4:SetValue(db.timerColor['a'])
		IB2OptionsTimer.SL5:SetValue(db.timerColor['r'])
		IB2OptionsTimer.SL6:SetValue(db.timerColor['g'])
		IB2OptionsTimer.SL7:SetValue(db.timerColor['b'])
		IB2OptionsTimer.T5:SetTextColor( IB:FormatColor(timerColor) )
		if db.timerDecimal then IB2OptionsTimer.CB1:SetChecked(true) else IB2OptionsTimer.CB1:SetChecked(false) end
		if db.timerHide then IB2OptionsTimer.CB2:SetChecked(true) else IB2OptionsTimer.CB2:SetChecked(false) end
		--if db.omniCC then IB2OptionsTimer.CB3:SetChecked(true) else IB2OptionsTimer.CB3:SetChecked(false) end
		if db.spiralHide then IB2OptionsTimer.CB4:SetChecked(true) else IB2OptionsTimer.CB4:SetChecked(false) end
	end

	-- Title text
	IB2OptionsTimer.T1 = CreateText(L['InterruptBar2 Timer Settings'], IB2OptionsTimer, 20,'TOP', -162, -20)

	-- Test button
	IB2OptionsTimer.BTN1 = CreateButton(L['Test'], IB2OptionsTimer, 'BOTTOM', 250, 13)
	IB2OptionsTimer.BTN1:SetScript('OnClick', function()
		for _,btn in pairs(btns) do
			IB:Deactivate(btn,GetTime())
			IB:Activate(btn,GetTime())
		end
	end)

	-- Text 2
	IB2OptionsTimer.T2 = CreateText(L['Font Settings'], IB2OptionsTimer, 12, 'TOP', -258, -70)

	-- Text Size
	IB2OptionsTimer.SL1 = CreateSlider(L['Text size'], IB2OptionsTimer, 7, 35, 1, db.timerScale, 'LEFT', 30, 165)
	IB2OptionsTimer.SL1:SetScript('OnValueChanged', function(self, value)
		_G[self:GetName() .. 'Text']:SetText(L['Text size'] .. ' (' .. floor(value) .. ')')
		db.timerScale = tonumber(value)
		IB:Restart()
	end)

	-- Hide decimals
	IB2OptionsTimer.CB1 = CreateCheckBox(L['Hide decimals'], IB2OptionsTimer, db.timerDecimal, 'TOP', -257, -185)
	IB2OptionsTimer.CB1:SetScript('OnClick', function()
		db.timerDecimal = not db.timerDecimal
		--IB:Restart()
	end)

	-- Hide text
	IB2OptionsTimer.CB2 = CreateCheckBox(L['Disable text'], IB2OptionsTimer.CB1, db.timerHide, 'LEFT', 180, 0)
	IB2OptionsTimer.CB2:SetScript('OnClick', function()
		db.timerHide = not db.timerHide
		for _,btn in pairs(btns) do
			--IB:Deactivate(btn)
			btn:Hide()
			btn:SetParent(nil)
		end
		wipe(btns)
		IB:CreateBar()
	end)

	-- OmniCC (changed to blizzards version instead)
	--[[IB2OptionsTimer.CB3 = CreateCheckBox(L['Enable Show numbers for Cooldowns (Blizzard)'], IB2OptionsTimer, db.omniCC,'TOP', -77,-140)
	IB2OptionsTimer.CB3:SetScript('OnClick', function()
		db.omniCC = not db.omniCC
		IB:Restart()
	end)]]

	-- Cooldown Spiral
	IB2OptionsTimer.CB4 = CreateCheckBox(L['Disable cooldown spiral'], IB2OptionsTimer, db.spiralHide,'LEFT', 350, 88)
	IB2OptionsTimer.CB4:SetScript('OnClick', function()
		db.spiralHide = not db.spiralHide
		IB:Restart()
	end)

	-- Text 4
	IB2OptionsTimer.T5 = CreateText(L['Text Position'], IB2OptionsTimer, 12, 'LEFT', 30, 40)

	-- Text Offset X
	IB2OptionsTimer.SL2 = CreateSlider(L['Text Offset X'], IB2OptionsTimer, -300, 300, 1, db.timerOffsetX, 'LEFT', 40, 0)
	IB2OptionsTimer.SL2:SetScript('OnValueChanged', function(self, value)
		_G[self:GetName() .. 'Text']:SetText(L['Text Offset X'] .. ' (' .. floor(value) .. ')')
		db.timerOffsetX = tonumber(value)
		IB:Restart()
	end)

	-- Text Offset Y
	IB2OptionsTimer.SL3 = CreateSlider(L['Text Offset Y'], IB2OptionsTimer.SL2, -300, 300, 1, db.timerOffsetY, 'LEFT', 175,0)
	IB2OptionsTimer.SL3:SetScript('OnValueChanged', function(self, value)
		_G[self:GetName() .. 'Text']:SetText(L['Text Offset Y'] .. ' (' .. floor(value) .. ')')
		db.timerOffsetY = tonumber(value)
		IB:Restart()
	end)

	-- Text color
	IB2OptionsTimer.T4 = CreateText(L['Text Color'], IB2OptionsTimer, 12, 'LEFT', 40, -65)
	IB2OptionsTimer.T4:SetTextColor( IB:FormatColor(db.timerColor) )

	-- Text transparent
	IB2OptionsTimer.SL4 = CreateSlider(L['Alpha'], IB2OptionsTimer, 0.1, 1, .1, db.timerColor['a'], 'LEFT', 40, -230)
	IB2OptionsTimer.SL4:SetWidth(100)
	IB2OptionsTimer.SL4:SetScript('OnValueChanged', function(self, value)
		db.timerColor['a'] = tonumber(value)
		IB:Restart()
		IB2OptionsTimer.T4:SetTextColor( IB:FormatColor(db.timerColor) )
	end)

	-- Red
	IB2OptionsTimer.SL5 = CreateSlider(L['Red'], IB2OptionsTimer, 0, 1, .1, db.timerColor['r'],'LEFT', 40, -110)
	IB2OptionsTimer.SL5:SetWidth(100)
	IB2OptionsTimer.SL5:SetScript('OnValueChanged', function(self, value)
		db.timerColor['r'] = tonumber(value)
		IB2OptionsTimer.T4:SetTextColor( IB:FormatColor(db.timerColor) )
		IB:Restart()
	end)

	-- Green
	IB2OptionsTimer.SL6 = CreateSlider(L['Green'], IB2OptionsTimer, 0, 1, .1, db.timerColor['g'], 'LEFT', 40, -150)
	IB2OptionsTimer.SL6:SetWidth(100)
	IB2OptionsTimer.SL6:SetScript('OnValueChanged', function(self, value)
		db.timerColor['g'] = tonumber(value)
		IB2OptionsTimer.T4:SetTextColor( IB:FormatColor(db.timerColor) )
		IB:Restart()
	end)

	-- Blue
	IB2OptionsTimer.SL7 = CreateSlider(L['Blue'], IB2OptionsTimer, 0, 1, .1, db.timerColor['b'], 'LEFT', 40, -190)
	IB2OptionsTimer.SL7:SetWidth(100)
	IB2OptionsTimer.SL7:SetScript('OnValueChanged', function(self, value)
		db.timerColor['b'] = tonumber(value)
		IB2OptionsTimer.T4:SetTextColor( IB:FormatColor(db.timerColor) )
		IB:Restart()
	end)

	-- << ANNOUNCEMENTS  & COMBAT >>
	local IB2OptionsAnnounce = CreateFrame('Frame')
	IB2OptionsAnnounce.name = L['Combat']
	IB2OptionsAnnounce.parent = IB2Options.name
	IB2OptionsAnnounce.refresh = function()
		if db.announcePartyuse then IB2OptionsAnnounce.CB1:SetChecked(true) else IB2OptionsAnnounce.CB1:SetChecked(false) end
		if db.announcePartyfinish then IB2OptionsAnnounce.CB2:SetChecked(true) else IB2OptionsAnnounce.CB2:SetChecked(false) end
		if db.announcePartyless then IB2OptionsAnnounce.CB3:SetChecked(true) else IB2OptionsAnnounce.CB3:SetChecked(false) end
		if db.resetAbilities then IB2OptionsAnnounce.CB5:SetChecked(true) else IB2OptionsAnnounce.CB5:SetChecked(false) end
		if db.abilityMiss then IB2OptionsAnnounce.CB7:SetChecked(true) else IB2OptionsAnnounce.CB7:SetChecked(false) end
		if db.abilityRestart then IB2OptionsAnnounce.CB8:SetChecked(true) else IB2OptionsAnnounce.CB8:SetChecked(false) end
	end

	-- Title
	IB2OptionsAnnounce.T1 = CreateText(L['InterruptBar2 Combat Settings'], IB2OptionsAnnounce, 20,'TOP',-150,-20)

	-- Text 2
	IB2OptionsAnnounce.T2 = CreateText(L['Announce in party when:'], IB2OptionsAnnounce, 12,'LEFT',30,200)

	IB2OptionsAnnounce.CB1 = CreateCheckBox(L['Ability used'], IB2OptionsAnnounce, db.announcePartyuse,'LEFT',40,170)
	IB2OptionsAnnounce.CB1:SetScript('OnClick', function()
		db.announcePartyuse = not db.announcePartyuse
	end)

	IB2OptionsAnnounce.CB2 = CreateCheckBox(L['Cooldown finished'], IB2OptionsAnnounce, db.announcePartyfinish,'LEFT',40,150)
	IB2OptionsAnnounce.CB2:SetScript('OnClick', function()
		db.announcePartyfinish = not db.announcePartyfinish
	end)

	IB2OptionsAnnounce.CB3 = CreateCheckBox(L['Cooldown less than 5sec'], IB2OptionsAnnounce, db.announcePartyless,'LEFT',40,130)
	IB2OptionsAnnounce.CB3:SetScript('OnClick', function()
		db.announcePartyless = not db.announcePartyless
	end)

	-- Combat log filters
	IB2OptionsAnnounce.T3 = CreateText(L['Combat Filters'], IB2OptionsAnnounce, 15,'LEFT', 40,50)

	IB2OptionsAnnounce.CB5 = CreateCheckBox(L['Reset cooldown(s) when Prep/Snap/CotE is used'], IB2OptionsAnnounce, db.resetAbilities,'LEFT',40,10)
	IB2OptionsAnnounce.CB5:SetScript('OnClick', function()
		db.resetAbilities = not db.resetAbilities
	end)

	IB2OptionsAnnounce.CB7 = CreateCheckBox(L['Show red overlay on abilities that misses'], IB2OptionsAnnounce, db.abilityMiss,'LEFT',40,-30)
	IB2OptionsAnnounce.CB7:SetScript('OnClick', function()
		db.abilityMiss = not db.abilityMiss
	end)

	IB2OptionsAnnounce.CB8 = CreateCheckBox(L['Restart timer if same spell is used again, and timer is still active'], IB2OptionsAnnounce, db.abilityRestart,'LEFT',40,-70)
	IB2OptionsAnnounce.CB8:SetScript('OnClick', function()
		db.abilityRestart = not db.abilityRestart
	end)

	-------------
	local IB2OptionsTimerTab = CreateFrame('Frame')
	IB2OptionsTimerTab.name = L['Timer Events']
	IB2OptionsTimerTab.parent = IB2Options.name

	IB2OptionsTimerTab.refresh = function()
		IB2OptionsTimerTab.TC:SetTextColor( IB:FormatColor(timerColorFiveLeft) )
		IB2OptionsTimerTab.SL1:SetValue(db.barTransparencyFiveLeft)
		IB2OptionsTimerTab.SL2:SetValue(db.timerColorFiveLeft['a'])
		IB2OptionsTimerTab.SL3:SetValue(db.timerScaleFiveLeft)
		IB2OptionsTimerTab.SL4:SetValue(db.timerColorFiveLeft['r'])
		IB2OptionsTimerTab.SL5:SetValue(db.timerColorFiveLeft['g'])
		IB2OptionsTimerTab.SL6:SetValue(db.timerColorFiveLeft['b'])
		IB2OptionsTimerTab.TC2:SetTextColor( IB:FormatColor(timerColorMinute) )
		IB2OptionsTimerTab.SL7:SetValue(db.barTransparencyMinute)
		IB2OptionsTimerTab.SL8:SetValue(db.timerColorMinute['a'])
		IB2OptionsTimerTab.SL9:SetValue(db.timerScaleMinute)
		IB2OptionsTimerTab.SL10:SetValue(db.timerColorMinute['r'])
		IB2OptionsTimerTab.SL11:SetValue(db.timerColorMinute['g'])
		IB2OptionsTimerTab.SL12:SetValue(db.timerColorMinute['b'])
	end

	-- Header Text
	IB2OptionsTimerTab.T1 = CreateText(L['InterruptBar2 Timer Events'], IB2OptionsTimerTab, 20, 'TOP', -162,-20)

	-- Text 2
	IB2OptionsTimerTab.T2 = CreateText(L['5 seconds left on Timer'], IB2OptionsTimerTab, 12,'LEFT',30,180)

	IB2OptionsTimerTab.TC = CreateText('5', IB2OptionsTimerTab, 50,'LEFT',75,45)
	IB2OptionsTimerTab.TC:SetTextColor( IB:FormatColor(db.timerColorFiveLeft) )

	-- Icon opacity
	IB2OptionsTimerTab.SL1 = CreateSlider(L['5: Icon Opacity'], IB2OptionsTimerTab, .2, 1, .1, db.barTransparencyFiveLeft,'LEFT',40,145)
	IB2OptionsTimerTab.SL1:SetWidth(100)
	IB2OptionsTimerTab.SL1:SetScript('OnValueChanged', function(self, value)
		_G[self:GetName() .. 'Text']:SetText(L['5: Icon Opacity'] .. ' (' .. value .. ')')
		db.barTransparencyFiveLeft = tonumber(value)
		--IB:Restart()
	end)

	-- text alpha
	IB2OptionsTimerTab.SL2 = CreateSlider(L['5: Alpha'], IB2OptionsTimerTab, 0, 1, .1, db.timerColorFiveLeft['a'],'LEFT',40,-150)
	IB2OptionsTimerTab.SL2:SetWidth(100)
	IB2OptionsTimerTab.SL2:SetScript('OnValueChanged', function(self, value)
		_G[self:GetName() .. 'Text']:SetText(L['5: Alpha'] .. ' (' .. floor(value) .. ')')
		db.timerColorFiveLeft['a'] = tonumber(value)
		IB2OptionsTimerTab.TC:SetTextColor( IB:FormatColor(db.timerColorFiveLeft) )
		IB:Restart()
	end)

	-- Text size
	IB2OptionsTimerTab.SL3 = CreateSlider(L['5: Text Size'], IB2OptionsTimerTab, 1, 50, 1, db.timerScaleFiveLeft,'LEFT',40,100)
	IB2OptionsTimerTab.SL3:SetWidth(100)
	IB2OptionsTimerTab.SL3:SetScript('OnValueChanged', function(self, value)
		_G[self:GetName() .. 'Text']:SetText(L['5: Text Size'] .. ' (' .. floor(value) .. ')')
		db.timerScaleFiveLeft = tonumber(value)
		IB:Restart()
	end)

	-- Red
	IB2OptionsTimerTab.SL4 = CreateSlider(L['5: Red'], IB2OptionsTimerTab, 0, 1, .1, db.timerColorFiveLeft['r'],'LEFT',40,0)
	IB2OptionsTimerTab.SL4:SetWidth(100)
	IB2OptionsTimerTab.SL4:SetScript('OnValueChanged', function(self, value)
		db.timerColorFiveLeft['r'] = tonumber(value)
		IB2OptionsTimerTab.TC:SetTextColor( IB:FormatColor(db.timerColorFiveLeft) )
		IB:Restart()
	end)

	-- Green
	IB2OptionsTimerTab.SL5 = CreateSlider(L['5: Green'], IB2OptionsTimerTab, 0, 1, .1, db.timerColorFiveLeft['g'],'LEFT',40,-50)
	IB2OptionsTimerTab.SL5:SetWidth(100)
	IB2OptionsTimerTab.SL5:SetScript('OnValueChanged', function(self, value)
		db.timerColorFiveLeft['g'] = tonumber(value)
		IB2OptionsTimerTab.TC:SetTextColor( IB:FormatColor(db.timerColorFiveLeft) )
		IB:Restart()
	end)

	-- Blue
	IB2OptionsTimerTab.SL6 = CreateSlider(L['5: Blue'], IB2OptionsTimerTab, 0, 1, .1, db.timerColorFiveLeft['b'],'LEFT',40,-100)
	IB2OptionsTimerTab.SL6:SetWidth(100)
	IB2OptionsTimerTab.SL6:SetScript('OnValueChanged', function(self, value)
		db.timerColorFiveLeft['b'] = tonumber(value)
		IB2OptionsTimerTab.TC:SetTextColor( IB:FormatColor(db.timerColorFiveLeft) )
		IB:Restart()
	end)

	-- Text 3
	IB2OptionsTimerTab.T3 = CreateText(L['60+ seconds on Timer'], IB2OptionsTimerTab, 12,'LEFT',275,180)

	IB2OptionsTimerTab.TC2 = CreateText('4:20', IB2OptionsTimerTab, 50,'LEFT',290,45)
	IB2OptionsTimerTab.TC2:SetTextColor( IB:FormatColor(db.timerColorMinute) )

	-- Icon opacity
	IB2OptionsTimerTab.SL7 = CreateSlider(L['60: Icon Opacity'], IB2OptionsTimerTab, .2, 1, .1, db.barTransparencyMinute,'LEFT',275,145)
	IB2OptionsTimerTab.SL7:SetWidth(100)
	IB2OptionsTimerTab.SL7:SetScript('OnValueChanged', function(self, value)
		_G[self:GetName() .. 'Text']:SetText(L['60: Icon Opacity'] .. ' (' .. floor(value) .. ')')
		db.barTransparencyMinute = tonumber(value)
		IB:Restart()
	end)

	-- Text alpha
	IB2OptionsTimerTab.SL8 = CreateSlider(L['60: Alpha'], IB2OptionsTimerTab, 0, 1, .1, db.timerColorMinute['a'],'LEFT',275,-150)
	IB2OptionsTimerTab.SL8:SetWidth(100)
	IB2OptionsTimerTab.SL8:SetScript('OnValueChanged', function(self, value)
		db.timerColorMinute['a'] = tonumber(value)
		IB2OptionsTimerTab.TC2:SetTextColor( IB:FormatColor(db.timerColorMinute) )
		IB:Restart()
	end)

	-- Text size
	IB2OptionsTimerTab.SL9 = CreateSlider(L['60: Text Size'], IB2OptionsTimerTab, 1, 50, 1, db.timerScaleMinute,'LEFT',275,100)
	IB2OptionsTimerTab.SL9:SetWidth(100)
	IB2OptionsTimerTab.SL9:SetScript('OnValueChanged', function(self, value)
		db.timerScaleMinute = tonumber(value)
		IB:Restart()
	end)

	-- Red
	IB2OptionsTimerTab.SL10 = CreateSlider(L['60: Red'], IB2OptionsTimerTab, 0, 1, .1, db.timerColorMinute['r'],'LEFT',275,0)
	IB2OptionsTimerTab.SL10:SetWidth(100)
	IB2OptionsTimerTab.SL10:SetScript('OnValueChanged', function(self, value)
		db.timerColorMinute['r'] = tonumber(value)
		IB2OptionsTimerTab.TC2:SetTextColor( IB:FormatColor(db.timerColorMinute) )
		IB:Restart()
	end)

	-- Green
	IB2OptionsTimerTab.SL11 = CreateSlider(L['60: Green'], IB2OptionsTimerTab, 0, 1, .1, db.timerColorMinute['g'],'LEFT',275,-50)
	IB2OptionsTimerTab.SL11:SetWidth(100)
	IB2OptionsTimerTab.SL11:SetScript('OnValueChanged', function(self, value)
		db.timerColorMinute['g'] = tonumber(value)
		IB2OptionsTimerTab.TC2:SetTextColor( IB:FormatColor(db.timerColorMinute) )
		IB:Restart()
	end)

	-- Blue
	IB2OptionsTimerTab.SL12 = CreateSlider(L['60: Blue'], IB2OptionsTimerTab, 0, 1, .1, db.timerColorMinute['b'],'LEFT',275,-100)
	IB2OptionsTimerTab.SL12:SetWidth(100)
	IB2OptionsTimerTab.SL12:SetScript('OnValueChanged', function(self, value)
		db.timerColorMinute['b'] = tonumber(value) 
		IB2OptionsTimerTab.TC2:SetTextColor( IB:FormatColor(db.timerColorMinute) )
		IB:Restart()
	end)

	-- << PROFILES >>
	local IB2OptionsProfiles = CreateFrame('Frame')
	IB2OptionsProfiles.name = L['Profiles']
	IB2OptionsProfiles.parent = IB2Options.name
	IB2OptionsProfiles.refresh = function()
		--
	end

	-- Header Text
	IB2OptionsProfiles.T1 = CreateText(L['InterruptBar2 Profiles'], IB2OptionsProfiles, 20,'TOP',-195,-20)

	-- Text 2
	IB2OptionsProfiles.T3 = CreateText(L['Reset your current profile'], IB2OptionsProfiles, 12,'LEFT',40,200)

	IB2OptionsProfiles.BTN2 = CreateButton(L['Default'], IB2OptionsProfiles, 'LEFT',40,160)
	IB2OptionsProfiles.BTN2:SetWidth(175)
	IB2OptionsProfiles.BTN2:SetScript('OnClick', function()
		InterruptBar2DB[playerName] = defaultSettings
		ReloadUI()
	end)

	-- Text 3
	IB2OptionsProfiles.T4 = CreateText(L['Current profiles (Click to change active profile)'], IB2OptionsProfiles, 12,'LEFT',40,90)

	-- Temporary method
	local x,y = 40, 60
	for k,v in pairs(InterruptBar2DB) do
		if k and k ~= playerName then
			IB2OptionsProfiles.BTN3 = CreateButton(k, IB2OptionsProfiles, 'LEFT',x,y)
			IB2OptionsProfiles.BTN3:SetWidth(175)
			IB2OptionsProfiles.BTN3:SetScript('OnClick', function()
				InterruptBar2DB[playerName] = v
				IB:SavedVariables()
				IB:Restart()
			end)
			y = y - 50
			if y == -340 then
				x = x + 190
				y = 60
			end
		end
	end


	for _,name in ipairs({IB2OptionsTimer,IB2OptionsTimerTab,IB2OptionsAnnounce,IB2OptionsSpell,IB2OptionsProfiles}) do
		InterfaceOptions_AddCategory(name)
	end
end

----------------------------------
-- Options End
----------------------------------





----------------------------------
-- Startup / Events
----------------------------------

-- Initialize everything
function IB:PLAYER_LOGIN()
	IB:RegisterEvent('PLAYER_ENTERING_WORLD')
	IB:SavedVariables()
	IB:CreateBar()

	IB2Options = CreateFrame('Frame')
	IB2Options.name = L['InterruptBar2']

	IB2Options.BTN0 = CreateButton(L['Load Options'], IB2Options, 'LEFT', 40, 250)
	IB2Options.BTN0:SetWidth(175)
	IB2Options.BTN0:SetScript('OnClick', function()
		IB:LoadOptions()
		IB2Options.BTN0:Hide()
		InterfaceAddOnsList_Update()
	end)
	InterfaceOptions_AddCategory(IB2Options)

	SLASH_InterruptBar1 = '/ib'
	SlashCmdList['InterruptBar'] = function()
		InterfaceOptionsFrame_OpenToCategory(L['InterruptBar2'])
		InterfaceOptionsFrameOkay:Click('button', down)
		InterfaceOptionsFrame_OpenToCategory(L['InterruptBar2'])
	end
end

-- Check if in world/bg/arena
function IB:ZoneCheck(instance)
	local zoneType

	if ( instance == 'pvp' ) then
		zoneType = 1 -- BattleGround
	elseif ( instance == 'arena' ) then
		zoneType = 2 -- Arena
	end
	if ( zoneType == nil ) then
		zoneType = 0 -- World
	end
	return zoneType
end

--
function IB:Disable()
	IB:UnregisterEvent('COMBAT_LOG_EVENT_UNFILTERED')
	IB:UnregisterEvent('ARENA_PREP_OPPONENT_SPECIALIZATIONS')
	for _,btn in pairs(btns) do
		IB:Deactivate(btn)
		btn:Hide()
		btn:SetParent(nil)
	end
	wipe(btns)
end

--
function IB:PLAYER_ENTERING_WORLD(instance)
	local zoneType = IB:ZoneCheck(instance)
	wipe(classDetected)

	if ( zoneType == 1 ) and not ( db.zoneBattle ) or ( db.zoneArena ) and ( zoneType ~= 2 ) then
		IB:Disable()
	else
		-- Run
		IB:RegisterEvent('COMBAT_LOG_EVENT_UNFILTERED')
		if ( zoneType == 2 ) and ( db.smartArena ) then
			IsInArena = true
			IB:Disable()
			IB:RegisterEvent('ARENA_PREP_OPPONENT_SPECIALIZATIONS')
		else
			IsInArena = false
			IB:Restart()
		end
	end
end

-- Event Handler
IB:RegisterEvent('PLAYER_LOGIN')
IB:SetScript('OnEvent', function(self, event, ...)
	if ( event == 'COMBAT_LOG_EVENT_UNFILTERED' ) then
		IB:CLEU(event, ...)
		return
	elseif ( event == 'PLAYER_LOGIN' ) then
		IB:PLAYER_LOGIN()
		IB:UnregisterEvent('PLAYER_LOGIN')
		return
	elseif ( event == 'PLAYER_ENTERING_WORLD' ) then
		local _,instance = IsInInstance()
		IB:PLAYER_ENTERING_WORLD(instance)
		return
	elseif ( event == 'ARENA_PREP_OPPONENT_SPECIALIZATIONS' ) then
		IB:DetectArena()
	end
end)
